<?php
// Complete listing
namespace net\xmp\phpbook;

/**
 * Converts a number to the hexadecimal format.
 * @param int $dec Number
 */
function dec2hex($dec) {
    return '0x' . dechex($dec);
}

/**
 * Returns additional information for known OIDs (Object Identifier).
 * @param string $oid OID as string
 */
function oidInfo($oid) {
    static $info = array(
        '2.5.5.1' => 'Distinguished Name (DN)',
        '2.5.5.2' => 'OID',
        '2.5.5.3' => 'Case sensitive string',
        '2.5.5.4' => 'Case insensitive string',
        '2.5.5.5' => 'Printable string / IA5 String',
        '2.5.5.6' => 'Numeric string',
        '2.5.5.7' => 'DN with octet string / OR name',
        '2.5.5.8' => 'Boolean',
        '2.5.5.9' => 'Integer / Enumeration',
        '2.5.5.10' => 'Octet string',
        '2.5.5.11' => 'Time string (UTC)',
        '2.5.5.12' => 'Unicode string',
        '2.5.5.13' => 'OSI presentation address',
        '2.5.5.14' => 'DN with Unicode string',
        '2.5.5.15' => 'NT security descriptor',
        '2.5.5.16' => 'Large integer / Interval',
        '2.5.5.17' => 'SID',
    );
    return empty($info[$oid]) ? $oid : "$oid ($info[$oid])";
}

/**
 * Converts the binary GUID to a string
 * @param string $guid binary GUID
 */
function guid2str($guid) {
    $last = substr(bin2hex($guid), 20);
    $x = unpack('Vhead/v2mid/nmid3', $guid);
    return sprintf("%08x-%04x-%04x-%04x-%s", $x['head'], $x['mid1'],
                                             $x['mid2'], $x['mid3'], $last);
}

/**
 * Converts the string to a binary GUID
 * @param string $guidstr GUID as string
 */
function str2guid($guidstr) {
    $v = explode('-', $guidstr);
    return pack('VvvnH*', hexdec($v[0]), hexdec($v[1]), hexdec($v[2]),
                          hexdec($v[3]), $v[4]);
}

/**
 * Converts the binary SID to a string
 * @param string $sid binary SID
 */
function sid2str($sid) {
    $v = unpack('Crev/Ccnt/Sx/Nauth/V*sub', $sid);
    $str = "S-$v[rev]-$v[auth]";
    for ($i = 1; $i <= $v['cnt']; $i++) {
        $str .= '-' . $v["sub$i"];
    }
    return $str;
}

/**
 * Converts the string to a binary SID
 * @param string $sidstr SID as string
 */
function str2sid($sidstr) {
    $v = explode('-', $sidstr);
    $cnt = count($v) - 3;
    $sid = pack('CCSN', $v[1], $cnt, 0, $v[2]);
    for ($i = 0; $i < $cnt; $i++) {
        $sid .= pack('V', $v[$i + 3]);
    }
    return $sid;
}

/**
 * Converts the Windows timestamp to a string
 * @param string $stamp Windows timestamp
 */
function winStamp2str($stamp) {
    if (!$stamp) {
        return 'unknown';
    } elseif ($stamp == '9223372036854775807') {
        return 'never';
    }
    $stamp = substr($stamp, 0, -7);         // 100ns -> s
    $stamp = bcsub($stamp, '11644473600');  // 1601 -> 1970
    return date('Y-m-d H:i:s O', $stamp);
}

/**
 * Converts the Unix time (time()) to a Windows timestamp
 * @param int $stamp Unix time (seconds since 1970)
 */
function time2winStamp($stamp) {
    if (!$stamp) {
        return '0';
    } elseif ($stamp == 0xffffffff) {
        return '9223372036854775807';
    }
    $stamp = bcadd($stamp, '11644473600');  // 1970 -> 1601
    return $stamp . '0000000';              // s ->  100ns
}

/**
 * Returns additional information for UserAccountControl.
 * @param int $ac Value of UserAccountControl
 */
function accountControl2str($ac) {
    // List of a few flags, not complete 
    static $flags = array(
        'ACCOUNTDISABLE' => 0x0002,
        'LOCKOUT' => 0x0010,
        'PASSWD_NOTREQD' => 0x0020,
        'PASSWD_CANT_CHANGE' => 0x0040,
        'NORMAL_ACCOUNT' => 0x0200,
        'INTERDOMAIN_TRUST_ACCOUNT' => 0x0800,
        'WORKSTATION_TRUST_ACCOUNT' => 0x1000,
        'SERVER_TRUST_ACCOUNT' => 0x2000,
        'DONT_EXPIRE_PASSWORD' => 0x10000,
        'PASSWORD_EXPIRED' => 0x800000);
    $info = array();
    foreach ($flags as $name => $value) {
        if ($ac & $value) {
            $info[] = $name;
        }
    }
    return dec2hex($ac) . ' (' . join(',  ', $info) . ')';
}

/**
 * Returns additional information for groupType.
 * @param int $type Value of the groupType attribute
 */
function groupType2str($type) {
    static $flags = array(
        'SECURITY_ENABLED' => 0x80000000,
        'ACCOUNT_GROUP' => 0x02,
        'RESOURCE_GROUP' => 0x04,
        'UNIVERSAL_GROUP' => 0x08,
    );
    $info = array();
    foreach ($flags as $name => $value) {
        if ($type & $value) {
            $info[] = $name;
        }
    }
    return dec2hex($type) . ' (' . join(',  ', $info) . ')';
}

/**
 * Mask the filter string for the search (conservative)
 * @param string $str Filter string to mask
 */
function escapeLDAP($str) {
    return preg_replace_callback('/[^-\w\s=,.*]/',
            function($m) {
                return '\\' . bin2hex($m[0]);
            },
            $str);
}
?>
